import { PrimitivesObject, PrimitiveType, ScalarObject, ScalarType, Table, TableDto } from "datapipe-js";
import { DbTypeConverter } from "./db-type-converter";
import { JoinType } from "./db-types";
export * from "./db-types";
/**
 * Interface that represents a SQL save result
 */
export interface SqlSaveStatus {
    /**
     * Number of inserted row
     */
    inserted: number;
    /**
     * Number of updated row
     */
    updated: number;
    /**
     * Number of deleted row
     */
    deleted: number;
}
/**
 * Sql Query configuration
 */
export interface SqlReadQueryInfo {
    /**
     * A list of comma separated fields to select.
     * Examples:
     *  - 'Field1, Field2'
     *  - 't.Field1, t.Field2' - where t is an alias
     *  - 'concat(FirstName, ' ', LastName) FullName, AnotherField' - you can use SQL functions
     *  - 'groupBy|Country, sum(profit) Profit' - aggregated query.
     */
    fields?: string;
    /**
     * A filter expression. Where you can use SQL functions, aliases and parameters
     * Examples:
     *  - "Country = 'UK'"
     *  - "Country = @country AND City = @city"
     *  - "concat(t.FirstName, ' ', t.LastName) = @fullName"
     */
    filter?: string;
    /**
     * A filter parameters used in an expresiion
     * e.g: {country: 'UK', fullName: 'Adam Smith'}
     */
    filterParams?: Record<string, ScalarType>;
    /**
     * Number of rows to skip
     */
    skip?: number;
    /**
     * top rows to take
     */
    top?: number;
    /**
     * Define a sortings fields
     */
    orderBy?: string;
    /**
     * Alias for main table. But default it is undefined
     */
    mainTableAlias?: string;
    /**
     * joins tables together
     * Examples:
     *  - ['InnerJoin', 'Customers c', 't.CustomerId = c.CustomerID'] - inner joins mainTable to Cutomers table
     */
    joins?: [JoinType, string, string, string?][];
}
/**
 * Sql Save operation config
 */
export interface SqlSaveOptions {
    /**
     * save types
     */
    method: "Merge" | "Append" | "BulkInsert";
    /**
     * a batch size for optimization point
     */
    batchSize?: number;
    /**
     * Define a primary key that should be used. Normally primary keys are taken from the table,
     * Use this property only if you want to upsert (merge) data on some other fields
     */
    primaryKeys?: string[];
    /**
     * Report progress on batch saving
     */
    batchProgressFunc?: (processedCount: number, status: SqlSaveStatus) => void;
}
export interface SqlQueryResponse {
    message?: string;
    table?: TableDto;
    outputParameters?: Record<string, any>;
    resultType: "Table" | "Items";
    items?: PrimitivesObject[];
}
/**
 * Set Base URL for SQL data api
 * @param baseUrl
 */
export declare function setBaseUrl(baseUrl: string): void;
/**
 * set bearer token for authentication
 * @param bearerToken
 */
export declare function setBearerToken(bearerToken: string): void;
/**
 * Set api Access Token issued by Worksheet System's API
 * @param userAccessToken
 */
export declare function setUserAccessToken(userAccessToken: string): void;
/**
 * Use Worksheet Systems' username and password to Authenticate for further api calls.
 * It logins you and set bearer token for further api calls
 */
export declare function authenticate(username: string, password: string): Promise<boolean>;
/**
 * A factory function that creates SqlDataApi instance
 * @param connectionName connectionName
 * @param config define baseUrl, apiAccessToken or bearer token
 * @returns
 */
export declare function sqlDataApi(connectionName: string, config?: {
    baseUrl?: string;
    userAccessToken?: string;
    bearerToken?: string;
}, abortSignal?: AbortSignal): SqlDataApi;
export declare function dbTypeConverter(): DbTypeConverter;
export declare function httpRequest<TRequest, TResponse>(method: "GET" | "POST" | "PUT" | "DELETE", url: string, body?: TRequest, config?: Record<string, any>): Promise<ServerResponse<TResponse>>;
export declare function httpGet<TResponse>(url: string, config?: Record<string, any>): Promise<TResponse>;
export declare function httpGetText(url: string, config?: Record<string, any>): Promise<string>;
export declare function httpPost<TRequest, TResponse>(url: string, body: TRequest, config?: Record<string, any>): Promise<TResponse>;
export declare function httpPut<TRequest, TResponse>(url: string, body: TRequest, config?: Record<string, any>): Promise<TResponse>;
export declare function httpDelete<TRequest, TResponse>(url: string, body: TRequest, config?: Record<string, any>): Promise<TResponse>;
/**
 *
 */
export declare class SqlDataApi {
    private baseUrl;
    private connectionName;
    private abortSignal?;
    private readonly userAccessToken?;
    private readonly bearerToken?;
    private queryInfo;
    private tableName;
    static BaseUrl: string;
    static UserAccessToken: string;
    static BearerToken: string;
    constructor(baseUrl: string, connectionName: string, config: {
        userAccessToken?: string;
        bearerToken?: string;
    }, abortSignal?: AbortSignal | undefined);
    /**
     * sets filter and returns same
     * - use this method for a fluent API
     * @param filter filter string e.g. "country='UK' or city=@city"
     * @param filterParams filter parameters e.g. {city: 'London'}
     * @returns same SqlDataApi instance
     */
    filter(filter: string, filterParams?: Record<string, ScalarType>): SqlDataApi;
    /**
     * Append filter (with AND) and returns same
     * - use this method for a fluent API
     * @param filter filter string e.g. "country='UK' or city=@city"
     * @param filterParams filter parameters e.g. {city: 'London'}
     * @returns same SqlDataApi instance
     */
    andFilter(filter: string, filterParams?: Record<string, ScalarType>): SqlDataApi;
    /**
     * sets fields that you want to setup
     * - use this method for a fluent API
     * @param fields - a a list of fields separated by comma
     * @returns same SqlDataApi instance
     */
    select(fields: string): SqlDataApi;
    orderBy(orderBy: string): SqlDataApi;
    top(top: number | string): SqlDataApi;
    innerJoin(tableName: string, joinCondition: string): SqlDataApi;
    leftJoin(tableName: string, joinCondition: string): SqlDataApi;
    rightJoin(tableName: string, joinCondition: string): SqlDataApi;
    join(joinType: JoinType, tableName: string, joinCondition: string, joinCondition2?: string): SqlDataApi;
    table(name: string): SqlDataApi;
    /**
     * queries data from SQL Server
     * @returns An array of items
     */
    query(tableOrViewName?: string, fieldsOrQuery?: string | SqlReadQueryInfo, queryInfoSettings?: SqlReadQueryInfo): Promise<ScalarObject[]>;
    /**
     * queries data from SQL table/view and returns results as a TableDto
     */
    queryToTable(tableOrViewName: string, fieldsOrQuery?: string | SqlReadQueryInfo, queryInfoSettings?: SqlReadQueryInfo): Promise<Table<PrimitiveType>>;
    setAbortSignal(signal?: AbortSignal): SqlDataApi;
    /**
     * Updates data in the table based on filter parameters
     * @returns Number of rows affected
     */
    updateData(tableName: string, updateData: Record<string, ScalarType>, filter?: string, filterParams?: Record<string, ScalarType>): Promise<number>;
    /**
     * Delete records from the table based on filter criteria
     */
    deleteFrom(tableName: string, filter?: string, filterParams?: Record<string, ScalarType>): Promise<number>;
    /**
     * Deletes rows from the table based on a primary keys. Only key fields have to be provided
     * @returns success
     */
    delete(tableName: string, items: Record<string, ScalarType>[]): Promise<boolean>;
    /**
     * Upsert(Merge), Append or BulkInsert an array of items into the table based on save options
     * If third parameter is an array, it will delete records from the table. Only Key Fields must be provided
     *  Example with a simple default options, that will merge array of items into the table:
     *   - sqlDataApi('someConnection').save('someTable', arrayOfItems)
     * @param tableName table Name
     * @param items an array of items to store. Only primary key fields are mandatory
     * @param itemsToDeleteOrSaveOptions items to delete or SaveOptions object
     * @param saveOptions  SaveOptions Object
     * @returns
     */
    save(tableName: string, items: ScalarObject[], itemsToDeleteOrSaveOptions?: Record<string, unknown>[] | SqlSaveOptions, saveOptions?: SqlSaveOptions): Promise<SqlSaveStatus>;
    /**
     * Saves a single record into the database and returns autogenerated ID field.
     * SQL Table should have Auto Indentity on one of the fields
     * @returns
     */
    saveWithAutoId(tableName: string, item: ScalarObject): Promise<number>;
    /**
     * Executes a SQL Query or stored procedure with parameters
     * @returns Raw result (SqlQueryResponse) with a table in it
     */
    sqlExecuteRaw(sql: string, params?: ScalarObject, paramDirections?: Record<string, string>): Promise<SqlQueryResponse>;
    /**
     * Executes a SQL Query or stored procedure with parameters
     * @returns result as a list of arrays
     */
    sqlExecute(sql: string, params?: ScalarObject): Promise<ScalarObject[] | unknown>;
    private saveData;
    private toPrimitive;
    private _query;
    private _queryTable;
}
interface ServerResponse<T> {
    data: T;
    isOk?: boolean;
    status: number;
    statusText: string;
    errorMessage?: string;
}
//# sourceMappingURL=sql-data.api.d.ts.map